@@ -1,15 +1,15 @@
-use std::{net::SocketAddr, ops::DerefMut, sync::Mutex};
+use std::{ops::DerefMut, sync::Mutex};
 
 use bevy::{
     app::{App, CoreStage, Plugin as PluginType},
     ecs::schedule::SystemStage,
     prelude::*,
 };
-
-use naia_client::{Client, ClientConfig, ProtocolType, Replicate, SharedConfig};
-
+use naia_client::{Client, ClientConfig, ProtocolType, SharedConfig};
 use naia_bevy_shared::WorldData;
 
+use crate::systems::should_receive;
+
 use super::{
     events::{
         DespawnEntityEvent, DisownEntityEvent, InsertComponentEvent, MessageEvent, NewCommandEvent,
@@ -24,56 +24,37 @@ use super::{
     },
 };
 
-struct PluginConfig<P: ProtocolType, R: Replicate<P>> {
+struct PluginConfig<P: ProtocolType> {
     client_config: ClientConfig,
     shared_config: SharedConfig<P>,
-    server_address: SocketAddr,
-    auth: Option<R>,
 }
 
-impl<P: ProtocolType, R: Replicate<P>> PluginConfig<P, R> {
-    pub fn new(
-        client_config: ClientConfig,
-        shared_config: SharedConfig<P>,
-        server_address: SocketAddr,
-        auth: Option<R>,
-    ) -> Self {
+impl<P: ProtocolType> PluginConfig<P> {
+    pub fn new(client_config: ClientConfig, shared_config: SharedConfig<P>) -> Self {
         PluginConfig {
             client_config,
             shared_config,
-            server_address,
-            auth,
         }
     }
 }
 
-pub struct Plugin<P: ProtocolType, R: Replicate<P>> {
-    config: Mutex<Option<PluginConfig<P, R>>>,
+pub struct Plugin<P: ProtocolType> {
+    config: Mutex<Option<PluginConfig<P>>>,
 }
 
-impl<P: ProtocolType, R: Replicate<P>> Plugin<P, R> {
-    pub fn new(
-        client_config: ClientConfig,
-        shared_config: SharedConfig<P>,
-        server_address: SocketAddr,
-        auth_ref: Option<R>,
-    ) -> Self {
-        let config = PluginConfig::new(client_config, shared_config, server_address, auth_ref);
+impl<P: ProtocolType> Plugin<P> {
+    pub fn new(client_config: ClientConfig, shared_config: SharedConfig<P>) -> Self {
+        let config = PluginConfig::new(client_config, shared_config);
         return Plugin {
             config: Mutex::new(Some(config)),
         };
     }
 }
 
-impl<P: ProtocolType, R: Replicate<P>> PluginType for Plugin<P, R> {
+impl<P: ProtocolType> PluginType for Plugin<P> {
     fn build(&self, app: &mut App) {
         let config = self.config.lock().unwrap().deref_mut().take().unwrap();
-        let mut client = Client::<P, Entity>::new(config.client_config, config.shared_config);
-
-        if let Some(auth) = config.auth {
-            client.auth(auth);
-        }
-        client.connect(config.server_address);
+        let client = Client::<P, Entity>::new(config.client_config, config.shared_config);
 
         app
         // RESOURCES //
@@ -96,26 +77,28 @@ impl<P: ProtocolType, R: Replicate<P>> PluginType for Plugin<P, R> {
             // events //
             .add_stage_before(CoreStage::PreUpdate,
                               PrivateStage::BeforeReceiveEvents,
-                              SystemStage::single_threaded())
+                              SystemStage::single_threaded()
+                                  .with_run_criteria(should_receive::<P>))
             .add_stage_after(PrivateStage::BeforeReceiveEvents,
-                              Stage::ReceiveEvents,
-                              SystemStage::single_threaded())
+                             Stage::ReceiveEvents,
+                             SystemStage::single_threaded()
+                                 .with_run_criteria(should_receive::<P>))
             .add_stage_after(PrivateStage::BeforeReceiveEvents,
                               Stage::Connection,
                               SystemStage::single_threaded()
-                                 .with_run_criteria(should_connect.system()))
+                                 .with_run_criteria(should_connect))
             .add_stage_after(Stage::Connection,
                               PrivateStage::AfterConnection,
                               SystemStage::parallel()
-                                 .with_run_criteria(should_connect.system()))
+                                 .with_run_criteria(should_connect))
             .add_stage_after(PrivateStage::BeforeReceiveEvents,
                               Stage::Disconnection,
                               SystemStage::single_threaded()
-                                 .with_run_criteria(should_disconnect.system()))
+                                 .with_run_criteria(should_disconnect))
             .add_stage_after(Stage::Disconnection,
                               PrivateStage::AfterDisconnection,
                               SystemStage::parallel()
-                                 .with_run_criteria(should_disconnect.system()))
+                                 .with_run_criteria(should_disconnect))
             // frame //
             .add_stage_after(CoreStage::PostUpdate,
                               Stage::PreFrame,
@@ -130,19 +113,19 @@ impl<P: ProtocolType, R: Replicate<P>> PluginType for Plugin<P, R> {
             .add_stage_after(Stage::PostFrame,
                               Stage::Tick,
                               SystemStage::single_threaded()
-                                 .with_run_criteria(should_tick.system()))
+                                 .with_run_criteria(should_tick))
             .add_stage_after(Stage::Tick,
                               PrivateStage::AfterTick,
                               SystemStage::parallel()
-                                 .with_run_criteria(should_tick.system()))
+                                 .with_run_criteria(should_tick))
             // SYSTEMS //
             .add_system_to_stage(PrivateStage::BeforeReceiveEvents,
                                  before_receive_events::<P>.exclusive_system())
             .add_system_to_stage(PrivateStage::AfterConnection,
-                                 finish_connect.system())
+                                 finish_connect)
             .add_system_to_stage(PrivateStage::AfterDisconnection,
-                                 finish_disconnect.system())
+                                 finish_disconnect)
             .add_system_to_stage(PrivateStage::AfterTick,
-                                 finish_tick.system());
+                                 finish_tick);
     }
 }
